----------------------------------------------------------------------
-- This file is ment to provide a layer of abstraction to buttons	--
-- FlexBar2 buttons do not use Action IDs, most code in this file	--
-- is aimed to emulate the behaviour of Action IDs.					--
----------------------------------------------------------------------

local Action =  FlexBar2:NewModule("Action", "AceEvent-2.0");
Action.Version = 1;

Action.ItemCache = {};
local ItemCache = Action.ItemCache;
function Action:OnEnable()
    self:BuildItemCache();
    self:RegisterEvent("BAG_UPDATE", "BuildItemCache");
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", function(UnitId) if(UnitId == "player") then Action:BuildItemCache(); end end);
end

function Action:BuildItemCache()
    for ItemId, _ in pairs(ItemCache) do
        for Info, _ in pairs(ItemCache[ItemId]) do
            ItemCache[ItemId][Info] = nil;
        end
    end
    for Bag = 0, 4 do
        for Slot =1, GetContainerNumSlots(Bag) do
            local ItemLink = GetContainerItemLink(Bag, Slot);
            if(ItemLink) then
                local _, _, ItemId = ItemLink:find("item:(%d+):");
                if(ItemId) then
                    local ItemId = "item:" .. ItemId;
                    if(type(ItemCache[ItemId]) ~= "table") then ItemCache[ItemId] = {}; end
                    ItemCache[ItemId].Bag = Bag;
                    ItemCache[ItemId].BagSlot = Slot;
                else
                    FlexBar2:Debug("Error with item:" , ItemId, ItemLink);
                end
            end
        end
    end
    for Slot = 0, 23 do
        local ItemLink = GetInventoryItemLink("player", Slot);
        if(ItemLink) then
            local _, _, ItemId = ItemLink:find("item:(%d+):");
            if(ItemId) then
                local ItemId = "item:" .. ItemId;
                if(type(ItemCache[ItemId]) ~= "table") then ItemCache[ItemId] = {}; end
                ItemCache[ItemId].InvSlot = Slot;
            end
        end
    end
end

Action.ButtonMixin = FlexBar2.OO.Mixin
{
    "LockAction",
    "UnlockAction",
	"ClearAction",
    "PickupAction",
	"PlaceAction",
	"GetActionTexture",
	"UpdateActionTexture",
    "UpdateCooldownSpinner",
    "PreClick_DisableButton",
    "PostClick_PickOldAction",
};

function Action.ButtonMixin:Activate()
    self.Frame:RegisterForDrag("LeftButton");
	self:RegisterScript("OnAttributeChanged", function(self, Trigger, Button, AttributeName, AttributeValue) if(AttributeName == "state-parent") then FlexBar2:Debug(type(AttributeValue)); self:UpdateActionTexture(); end end);
	self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateActionTexture");
	self:UpdateActionTexture();
    self.OldActionCache = {};
end

function Action.ButtonMixin:SetDefaults()
    self.InfoTable.LockAction = false;
	self:UpdateActionTexture();
end

function Action.ButtonMixin:LoadSettings()
    if(self.InfoTable.LockAction == true) then
        self:LockAction();
    else
        self:UnlockAction();
    end
end

function Action.ButtonMixin:LockAction()
    self:UnregisterScript("OnDragStart", "PickupAction");
    self:UnregisterScript("OnReceiveDrag", "PlaceAction");
    self:UnregisterScript("PreClick", "PreClick_DisableButton");
    self:UnregisterScript("PostClick", "PostClick_PickOldAction");
end

function Action.ButtonMixin:UnlockAction()
    self:RegisterScript("OnDragStart", "PickupAction");
    self:RegisterScript("OnReceiveDrag", "PlaceAction");
    self:RegisterScript("PreClick", "PreClick_DisableButton");
    self:RegisterScript("PostClick", "PostClick_PickOldAction");
end

function Action.ButtonMixin:GetActionTexture()
	local ActionType = self:GetModifiedAttribute("type");
	if(ActionType == "spell") then
		return GetSpellTexture(self:GetModifiedAttribute("spell")) or "";
	elseif(ActionType == "item") then
		local ItemName, ItemLink, ItemRarity, ItemLevel, ItemMinLevel, ItemType, ItemSubType, ItemStackCount, ItemEquipLoc, ItemTexture = GetItemInfo(self:GetModifiedAttribute("item"));
		return ItemTexture or "";
	elseif(ActionType == "macro") then
		local MacroName, MacroTexture, MacroBody, MacroLocal = GetMacroInfo(self:GetModifiedAttribute("macro"));
		return MacroTexture or "";
	else
		return "";
	end
end

function Action.ButtonMixin:UpdateActionTexture()
	self:SetIcon(self:GetActionTexture());
end

function Action.ButtonMixin:UpdateCooldownSpinner()
    
end

function Action.ButtonMixin:ClearAction()
    if(InCombatLockdown() == nil) then
        self:SetModifiedAttribute("type", nil);
        self:SetModifiedAttribute("spell", nil);
        self:SetModifiedAttribute("macro", nil);
        self:SetModifiedAttribute("item", nil);
        self:UpdateActionTexture();
    end
end

function Action.ButtonMixin:PreClick_DisableButton()
    if(GetCursorInfo() ~= nil) then        
        local OldActionCache = self.OldActionCache;
        OldActionCache.ActionType = self:GetModifiedAttribute("type");
        OldActionCache.ActionAttribute = self:GetModifiedAttribute(OldActionCache.ActionType);
        self.Frame:Disable();
        self:PlaceAction();
        self.Frame:Enable();
    end
end

function Action.ButtonMixin:PostClick_PickOldAction()
    local OldActionCache = self.OldActionCache;
    if(OldActionCache.ActionType ~= nil and OldActionCache.ActionAttribute ~= nil) then
        self:PickupAction(OldActionCache.ActionType, OldActionCache.ActionAttribute, true);
        OldActionCache.ActionType = nil;
        OldActionCache.ActionAttribute = nil;
    end
end

function Action.ButtonMixin:PickupAction(ForcedActionType, ForcedActionAttribute, DontTouchCurrentSetting)
	if(InCombatLockdown() == nil) then
        ClearCursor(); -- First thing to do, make sure the cursor is clear.
		local ActionType = ForcedActionType or self:GetModifiedAttribute("type");
        if(ActionType) then
            local ActionAttribute = ForcedActionAttribute or self:GetModifiedAttribute(ActionType);
            FlexBar2:Debug("PickupAction:", ActionType);
            if(ActionType == "spell") then
                PickupSpell(ActionAttribute);
            elseif(ActionType == "item") then
                local ItemInfo = ItemCache[ActionAttribute];
                FlexBar2:Debug("Item: ",ActionAttribute);
                if(type(ItemInfo) == "table") then
                    if(ItemInfo.Bag and ItemInfo.BagSlot) then
                        PickupContainerItem(ItemInfo.Bag, ItemInfo.BagSlot);
                    elseif(ItemInfo.InvSlot) then
                        PickupInventoryItem(ItemInfo.InvSlot);
                    else
                        FlexBar2:Print("Item currently not in the inventory, so can not pickup. Sorry :(");
                    end                
                else
                    FlexBar2:Print("Item currently not in the inventory, so can not pickup. Sorry :(");
                end
                
            elseif(ActionType == "macro") then
                PickupMacro(ActionAttribute);
            end
            if(not DontTouchCurrentSetting) then self:ClearAction(); end
        end
	end
end

function Action.ButtonMixin:PlaceAction()
    if(InCombatLockdown() == nil) then
        local ActionType, ActionId, ActionBook = GetCursorInfo()
		FlexBar2:Debug("PlaceAction:", ActionType, ActionId, ActionBook);
        if(ActionType ~= nil) then self:PickupAction(); end
        if(ActionType == "spell") then
            self:SetModifiedAttribute("type", "spell");
            local SpellName, SpellRank = GetSpellName(ActionId, ActionBook);
            if(type(SpellRank) == "string" and string.find(SpellRank, "^Rank %d+$")) then
                SpellName = SpellName .. "(" .. SpellRank .. ")";
            end
            self:SetModifiedAttribute("spell", SpellName);
        elseif(ActionType == "item") then
			self:SetModifiedAttribute("type", "item");
            self:SetModifiedAttribute("item", "item:" .. ActionId);
        elseif(ActionType == "macro") then
			local MacroName, MacroTexture, MacroBody, MacroLocal = GetMacroInfo(ActionId);
			self:SetModifiedAttribute("type", "macro");
			self:SetModifiedAttribute("macro", ActionId);
        end
		self:UpdateActionTexture();
    else
        FlexBar2:Print("Actions cannot be changed in combat, sorry");
    end
end
